<?php
#ini_set('display_errors', 1);
#error_reporting(E_ALL);
require_once 'class.interface.mycv.php';
require_once 'class.mycvparser.php';
require_once 'config.php';
require_once 'functions.php';

/**
 * A class for building CV pages.
 * @author Luis Leiva
 * @date 18 Jun 2010
 * @require class.mycvparser.php
 * @require functions.php
 */
class MyCV extends MyCVParser implements IMyCV
{
  /** Custom DOM elements in page head. */
  var $DOMElementsHead;
  /** Custom DOM elements in page body. */
  var $DOMElementsBody;
  /** Custom DOM elements in page navigation. */
  var $DOMElementsNav;
  /** Custom DOM elements in page footer. */
  var $DOMElementsFooter;
  /** Custom DOM elements in page global. */
  var $DOMElementsGlobal;
  
  /**
   * Renders the web page. Steps: validation, head elements, personal info, sections, and footer
   * @param string $cache Whether rendered page should be cached or not, since UPV server is slooow parsing PHP classes.
   */
  public function render($cache = false)
  {
    if( !$this->validate() ) die("Cannot validate CV file.");
    
    ob_start();
    // begin rendering ---------------------------------------------------------
    $this->pageStarts();
    $this->formatPersonal(); // if available
    $this->insertDOMElements( $this->DOMElementsNav ); // if available
    $this->formatSectionGroups();
    $this->pageEnds();
    // end of rendering --------------------------------------------------------
    $content = str_replace("\n", "", ob_get_contents());
    ob_end_clean();
    
    if ($cache) $this->saveCache($content);
    // finally display buffered content    
    echo $content;
  }
  
  /**
   * Applies a CSS theme
   * @param $dir  string  directory name, inside themes/ dir
   */
  public function setTheme($dir)
  {
    $theme = $this->loadFiles("themes", $dir);
    $this->setFiles($theme);
  }

  /**
   * Applies a plugin or a set of plugins
   * @param $dirs string  dir(s) name(s), inside plugins/ dir
   */
  public function setPlugins($load)
  {
    foreach ($load as $dir) {
      $plugins = $this->loadFiles("plugins", $dir);
      $this->setFiles($plugins);
    }
  }

  /**
   * Loads a widget or a set of widgets
   * @deprecated 
   * @param $dirs string  dir(s) name(s), inside widgets/ dir
   */
  public function setWidgets($load)
  {
    foreach ($load as $dir) {
      $widgets = $this->loadFiles("widgets", $dir, "class");
      $this->setFiles($widgets);
    }
  }

  /**
   * Load files from a dir and create the correspondent DOM item
   * @param $base string  base directory name, inside root dir
   * @param $dir  string  directory name, inside base dir
   */
  public function loadFiles($base, $dir, $prefix = "")
  {
    $files = array();
    $path = realpath(SYSPATH.'../'.$base.'/'.$dir);
    if ($handle = opendir($path)) {
      while (false !== ($file = readdir($handle))) 
      {
        $filepath = $path.'/'.$file;
        if (!is_file($filepath)) {
          continue;
        } else if (!empty($prefix)) {
          // load only those files starting by given prefix
          if (!str_startswith($file, $prefix)) continue;
        }
        $files[] = $base.'/'.$dir.'/'.$file; // $filepath
      }
      closedir($handle);
    }

    return $files;
  }

  /**
   * Load files from a dir and create the correspondent DOM item
   * @param $filepath string  path to file
   */
  public function setFiles(array $filepaths, $newest = TRUE)
  {
    $t = $newest ? '?'.time() : NULL;
    foreach ($filepaths as $filepath) {
      $ext = file_info($filepath, "extension");
      $uri = url_get_base();
      switch($ext) {
        case 'css':
          $this->addDOMElement('<link href="'.$uri.'/'.$filepath.$t.'" rel="stylesheet" type="text/css" />', "head");
          break;
        case 'js':
          // Note: We only defer script execution of JS plugins.
          $this->addDOMElement('<script src="'.$uri.'/'.$filepath.$t.'" type="text/javascript"></script>', "footer");
          break;
        default:
          break;
      }
    }
  }

  /**
   * Low-level function to create DOM elements, either in the head, body, or footer.
   * @param $elm mixed  HTML contents
   * @param $type string  head, body (default), nav, or footer
   */
  public function addDOMElement($elm, $type)
  {
    if (!$elm) return;

    switch($type) {
      case 'head':
      case 'header':
        $arr = &$this->DOMElementsHead;
        break;
      case 'nav':
        $arr = &$this->DOMElementsNav;
        break;
      case 'foot':
      case 'footer':
        $arr = &$this->DOMElementsFooter;
        break;
      case 'global':
        $arr = &$this->DOMElementsGlobal;
        break;
      case 'body':
      case 'aside':
      default:
        $arr = &$this->DOMElementsBody;
        break;
    }

    if (is_array($elm)) {
      foreach ($elm as $value) {
        $arr[] = $value;
      }
    } else {
      $arr[] = $elm;
    }
  }

  /**
   * Inserts page doctype.
   */    
  public function setDocType($version)
  {
    switch ($version) {
      case 'html4':
        $doc  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">';
        $doc .= '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">';
        break;
      case 'xhtml':
        $doc  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">';
        $doc .= '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">';
        break;
      case 'html5':
      default:
        $doc  = '<!DOCTYPE html>';
        $doc .= '<html lang="en">';
        break;
    }
    echo $doc;
  }

  /**
   * Inserts page metatags and basic elements.
   */   
  protected function setBasicHead(array $opts = null)
  {
    $h = array(
      // metatags
      '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />',
      '<meta http-equiv="Pragma" content="no-cache" />',
      '<meta http-equiv="Expires" content="-1" />',
      '<meta name="viewport" content="width=device-width, initial-scale=1.0" />',
#      '<meta name="Distribution" content="global" />',
#      '<meta name="Robots" content="index, follow" />',
      // favicon
#      '<link href="favicon.ico" rel="icon" type="image/x-icon" />',
      '<link href="favicon.ico" rel="shortcut icon" type="image/x-icon" />',
      // css
      '<link href="css/base.css" rel="stylesheet" type="text/css" async />',
      '<link href="css/bib.css" rel="stylesheet" type="text/css" async />',
      '<link href="css/ui.css" rel="stylesheet" type="text/css" async />',
      // scripts go at the bottom
    );
    if (is_array($opts)) {
      $h = array_merge($h, $opts);
    }
    echo implode("", $h);
  }   

  /**
   * Inserts doctype + head + metadata (+ custom head elements + custom body elements), and opens page body.
   */
  public function pageStarts()
  {
    $this->setDocType('html5');
    echo '<head>';
    $this->setBasicHead();
    $this->formatMetadata();
    $this->insertDOMElements( $this->DOMElementsHead );
    echo '</head>';
    echo '<body>';
    echo '<div id="global">';
    $this->insertDOMElements( $this->DOMElementsBody );
  }

  /**
   * Insert footer elements (if available) and closes page body.
   */
  public function pageEnds()
  {
    $this->insertDOMElements( $this->DOMElementsGlobal );
    echo '</div><!-- end global -->';
    echo '<script src="js/jquery-1.3.2.min.js" type="text/javascript"></script>';
    $this->insertDOMElements( $this->DOMElementsFooter );
    echo '</body>';
    echo '</html>';
  }

  /**
   * Insert custom DOM elements whenever they were defined (head, body, nav, or footer).
   */
  protected function insertDOMElements($e)
  {
    if ($e && count($e) > 0) {
      foreach ($e as $elm) {
        echo $elm;
      }
    }
  }

}
?>
