<?php
require_once 'class.mypublication.lite.php';
require_once 'functions.php';
require_once 'functions.bib.php';

/**
 * A class for parsing CV pages. Lite version.
 * @author Luis Leiva
 * @date 18 Jun 2010
 * @require class.mypublication.lite.php
 * @require functions.php
 * @require functions.bib.php
 */
class MyCVParser
{
  var $DOM;
  
  public function MyCVParser($file)
  {
    //libxml_use_internal_errors(true);
    $this->DOM = simplexml_load_file($file);
  }
  
  // properties (used var instead of const to override'em in subclasses) --------
  
  /** Regular expression to extract node contents. */
  var $REGEX_ENTRIES = "/[\r]?[\n]+/"; // split by newlines
  
  /** Separator to get URL and caption from images. */
  var $SEPARATOR_TABLE       = "||";
  /** Separator to get columns from tables. */
  var $SEPARATOR_IMAGE       = "%%";
  /** Separator to get items from definition lists. */
  var $SEPARATOR_DESCRIPTION = "##";
  
  /** Prefix for table captions. */
  var $CAPTION_PREFIX_TABLE  = "Table:";
  /** Prefix for image captions. */
  var $CAPTION_PREFIX_IMAGE = "Figure:";
  
  // methods -------------------------------------------------------------------

  public function formatPersonal()
  {
    print_line( '<div id="personal">' );
    $this->parsePersonal();
    print_line( '</div><!-- end personal -->' );
  }
  
  protected function parsePersonal()
  {
    $personal = $this->DOM->personal;
    foreach ($personal as $mydata)
    {
      $nodes = $mydata->children();
      foreach ($nodes as $node)
      {
        switch($node->getName())
        {
          case 'photo':
            $this->formatPhoto($node);
            break;
          case 'name':
            $this->formatName($node);
            break;
          case 'birthday':
            $this->formatBirthDay($node);
            break;
          case 'address':
            $this->formatAddress($node);
            break;
          case 'phone':
            $this->formatPhone($node);
            break;
          case 'email':
            $this->formatEmail($node);
            break;
          case 'webpage':
            $this->formatWebpage($node);
            break;
          case 'occupation':
          case 'affiliation':
          case 'notes':
            $this->formatText($node);
            break;
          default:
            break;
        }
      }
    }
  }

  public function getPhoto($node)
  {
    $img = $node['uri'];
    if (!empty($img)) {
      return '<img src="'.$img.'" alt="Photo" />';
    }
    
    return false;
  }
  
  public function formatPhoto($node)
  {
    print_line( '<div class="personal-photo">' );
    print_line( $this->getPhoto($node) );
    print_line( '</div><!-- end personal-photo -->' );
  }

  public function getName($node)
  {
    $entry = "";
    foreach ($node->attributes() as $attrName => $attrNodeValue) {
      if (!empty($attrNodeValue)) {
        $entry .= " " . $attrNodeValue;
      }
    }
    
    return trim($entry);
  }
  
  public function formatName($node)
  {
    print_line( '<div class="personal-name">' );
    print_line( $this->getName($node) );
    print_line( '</div><!-- end personal-name -->' );
  }

  public function getBirthDay($node)
  {
    // check typed month
    $month = (string)$node['month'];
    if (strlen($month) < 3) {
      $month = date('F', mktime(0,0,0,$month,1));
    }

    return trim( $node['day'] ." ". $month ." ". $node['year'] );
  }
  
  public function formatBirthDay($node)
  {
    print_line( '<div class="personal-birthday">' );
    print_line( $this->getBirthDay($node) );
    print_line( '</div><!-- end personal-birthday -->' );
  }

  public function getAddress($node)
  {
    $entry  = $node['street'];
    $entry .= " " . $node['zip'];
    $entry .= " &mdash; ";
    $entry .= $node['city'];
    $entry .= " (" . $node['country'] . ")";

    return $entry;
  }
  
  public function formatAddress($node)
  {
    print_line( '<div class="personal-address">' );
    print_line( $this->getAddress($node) );
    print_line( '</div><!-- end personal-address -->' );
  }

  public function getPhone($node)
  {
    $entry  = $node['prefix'];
    $entry .= " " . $node['number'];
    $extension = $node['extension'];
    if (!empty($extension)) {
      $entry .= " (ext. " . $extension . ")";
    }

    return $entry;
  }
  
  public function formatPhone($node)
  {
    
    print_line( '<div class="personal-phone">' );
    print_line( $this->getPhone($node) );
    print_line( '</div><!-- end personal-phone -->' );
  }

  public function getEmail($node)
  {
    $mail = $node['inbox'];
    // protect email from spammers
    $mail = str_replace("@", " AT_SYMBOL ", $mail);
    $mail = str_replace(".", " DOT ", $mail);
    
    return $mail;
  }
  
  public function formatEmail($node)
  {
    print_line( '<div class="personal-email">' );
    print_line( $this->getEmail($node) );
    print_line( '</div><!-- end personal-email -->' );
  }

  public function getWebpage($node)
  {
    $uri = url_check_protocol( $node['uri'] );
    
    $entry = "";
    if( $node['type'] ) {
      $entry .= '<span>' . $node['type'] . '</span>';
    }
    $entry .= ' <a href="'.$uri.'">' . $uri . '</a>';

    return $entry;
  }
  
  public function formatWebpage($node)
  {
    print_line( '<div class="personal-webpage">' );
    print_line( $this->getWebpage($node) );
    print_line( '</div><!-- end personal-webpage -->' );
  }

  public function getText($node)
  {
    $entry = "";
    $lines = $this->getEntries($node);
    foreach ($lines as $line) {
      $entry .= '<p>' . $line . '</p>';
    }

    return $entry;
  }
  
  public function formatText($node)
  {
    print_line( '<div class="personal-text">' );
    print_line( $this->getText($node) );
    print_line( '</div><!-- end personal-text -->' );
  }

  
  public function formatSectionGroups()
  {
    print_line( '<div id="sections">' );
    $this->parseSectionGroups();
    print_line( '</div><!-- end sections -->' );
  }
  
  protected function parseSectionGroups()
  {
    $sectiongroups = $this->DOM->sectiongroup;
    foreach ($sectiongroups as $sectiongroup)
    {
      print_line( '<fieldset class="section-group">' );
      print_line( '<legend>' . $sectiongroup['title'] . '</legend>' );

      $contents = $sectiongroup->children();
      foreach ($contents as $sectionindex => $node)
      {
        $name = $node->getName();
        
        // get sections
        if( $node['title'] )
        {
          $level = (int)$node['level'];
          switch ($name)
          {
            case 'section':
            default:
              $this->formatHeading($node, $level);
              break;

            case 'publications':
              $files = array();
              if ($node['include']) {
                $files['include'] = array_map('strtolower', explode(",", $node['include'])); 
              }
              if ($node['exclude']) {
                $files['exclude'] = array_map('strtolower', explode(",", $node['exclude']));
              }
              $this->formatPublications($node, $files);
              break;
          }
        }

        if ($name == 'text')
        {
          // just display raw node value
          print_line( '<div class="section-text">' );
          print_line( trim($node) );
          print_line( '</div><!-- end section-text -->' );
        }
        
        if( $name == 'env')
        {
          // apply some formatting
          $type = $node['type'];
          switch ($type)
          {
            case 'tabular':
              $this->formatTable($node);
              break;
            case 'itemize':
              $this->formatItemize($node);
              break;
            case 'enumerate':
              $this->formatEnumerate($node);
              break;
            case 'description':
              $this->formatDescription($node);
              break;
            case 'image':
              $this->formatImage($node);
            default:
              break;
          }
        }
      }
          
      print_line( '</fieldset><!-- end section-group -->' );
    }
  }
  

  /**
   * Applies format to image items
   * @param $node   XML node
   * @param $level int         Heading level (1, 2, 3, or 4)
   */
  public function formatHeading($node, $level)
  {
    $tag = 'h'.$level;
    $title = $node['title'];
    print_line( '<'.$tag.' class="section-heading">' . $title . '</'.$tag.'>' );
  }

  /**
   * Applies format to tabular items
   * @param $node  DOMElement  XML node
   */
  public function formatTable($node)
  {
    $hasCaption = false; // flag to better parse the table
    
    print_line( '<table class="environment env-table">' );
    $rows = $this->getEntries($node);
    foreach ($rows as $index => $row)
    {
      if ($index == 0 && strpos($row, $this->SEPARATOR_TABLE) === false)
      {
        $hasCaption = true;
        $entry = "";
        // check if caption was typed
        if ( (bool)$this->CAPTION_PREFIX_TABLE) {
          $entry .= '<caption><span>' . $this->CAPTION_PREFIX_TABLE .'</span> '. $row . '</caption>';
        } else /*if (!empty($row))*/ {
          $entry .= '<caption>' . $row . '</caption>';
        }
        print_line( $entry );
        continue;
      }
      
      $cols = explode($this->SEPARATOR_TABLE, $row);
      // first row are table headings
      if( ($hasCaption && $index == 1) || (!$hasCaption && $index == 0) ) {
        print_line( '<thead>' );
        print_line( '<tr>' );
        foreach ($cols as $col) {
          print_line( '<th>' . trim($col) . '</th>' );
        }
        print_line( '</tr>' );
        print_line( '</thead>' );
        continue;
      }
      // the rest are regular rows
      print_line( '<tbody>' );
      print_line( '<tr>' );
      foreach ($cols as $col) {
        print_line( '<td>' . trim($col) . '</td>' );
      }
      print_line( '</tr>' );
      print_line( '</tbody>' );
    }
    print_line( '</table>' );
  }

  /**
   * Applies format to itemize lists
   * @param $node  DOMElement  XML node
   */
  public function formatItemize($node)
  {
    print_line( '<ul class="environment env-itemize">' );
    $this->formatList($node);
    print_line( '</ul>' );
  }

  /**
   * Applies format to enumerate lists
   * @param $node  DOMElement  XML node
   */
  public function formatEnumerate($node)
  {
    print_line( '<ol class="environment env-enumerate">' );
    $this->formatList($node);
    print_line( '</ol>' );
  }

  /**
   * Applies format to description lists
   * @param $node  DOMElement  XML node
   */
  public function formatDescription($node)
  {
    print_line( '<ul class="environment env-description">' );
    $items = $this->getEntries($node);
    foreach ($items as $item) {
      list($dt,$dd) = explode($this->SEPARATOR_DESCRIPTION, $item);
      print_line( '<li><span>' . trim($dt) . '</span> ' . trim($dd) . '</li>' );
    }
    print_line( '</ul>' );
  }

  /**
   * Applies format to enumerate lists
   * @param $node  DOMElement  XML node
   */
  public function formatImage($node)
  {
    $items = $this->getEntries($node);
    // check if global caption should be typed
    if (count($items) > 2)
    {
      $globalcaption = "";
      if (strpos($items[0], $this->SEPARATOR_IMAGE) === false) {
        $globalcaption .= '<p class="caption-all">';
        if ( (bool)$this->CAPTION_PREFIX_IMAGE) {
          $globalcaption .= '<span>'.$this->CAPTION_PREFIX_IMAGE.'</span> ';
        }
        $globalcaption .= $items[0] . '</p>';
      }
      print_line( '<div class="environment env-image-wrap">' );
    }
    // now display images
    foreach ($items as $index => $item)
    {
      // check for errors
      if( (count($items) > 2 && $index == 0) /*|| strpos($item, $this->SEPARATOR_IMAGE) === false*/ ) continue;
      // get image parts
      list($uri,$caption) = explode($this->SEPARATOR_IMAGE, $item);
      // check for dot
      $dotpos = strpos($item, ".");
      if ($dotpos === false || $dotpos == strlen($item) - 1) continue;
      
      print_line( '<div class="environment env-image">' );
      print_line( '<img src="'.$uri.'" alt="Image" />' );
      if (!empty($caption))
      {
        $entry  = '<p class="caption">';
        if (count($items) > 2) {
          $entry .= '<span>('.$index.')</span> ';
        } else {
          if ( (bool)$this->CAPTION_PREFIX_IMAGE) {
            $entry .= '<span>'.$this->CAPTION_PREFIX_IMAGE.'</span> ';
          }
        }
        $entry .= $caption . '</p>';

        print_line( $entry );
      }
      print_line( '</div><!-- end env-image -->' );
    }
    
    if (isset($globalcaption)) {
      print_line( $globalcaption );
      print_line( '</div><!-- end env-image-wrap -->' );
    }
  }
  
  /**
   * Retrieves all BIB files from bib/ dir.
   * @param $files  array optional list of files to load
   */
  public function getPublications($files)
  {
    $bibs = array();
  	$dir = realpath(SYSPATH.'../bib/');
    if ($handle = opendir($dir)) {
      while (false !== ($file = readdir($handle))) {
        $ext = file_info($file, "extension");
        if ($ext == "bib")
        {
          $name = strtolower( file_info($file, "filename") );
          if (array_key_exists('include', $files) && count($files['include']) > 0) {
            // user want to *include* a set of BIB files
            if (!in_array($name, $files['include'])) continue;
          } else if (array_key_exists('exclude', $files) && count($files['exclude']) > 0) {
            // user want to *exclude* a set of BIB files (watch out in_array condition)
            if (in_array($name, $files['exclude'])) continue;
          }
          $bibs[] = $dir.'/'.$file;
        }
      }
      closedir($handle);
    }

    return $bibs;
  }

  /**
   * Formats retrieved BIB files
   * @param $bibs  array list of files to load
   * @param $wrapitem  string HTML element wrapper for each item (default: "div")
   */
  public function formatBibFiles($bibs, $wrapitem = "div")
  {
    foreach ($bibs as $bibfile) {
      $parse = new PARSEENTRIES();
      //$parse->expandMacro = true;
      $parse->openBib($bibfile);
      $parse->extractEntries();
      $parse->closeBib();
      list($preamble, $strings, $entries, $undefinedStrings) = $parse->returnArrays();
      // by now we only need the entries
      foreach ($entries as $entry) {
        print_line( '<'.$wrapitem.' class="bib-entry">' );
        $parsedbib = new MyPublication($entry);
        print_line( '</'.$wrapitem.'><!-- end bib-entry -->' );
      }
    }
  }

  /**
   * Applies format to publication items
   * @param $node  DOMElement  XML node
   * @param $files array  include or exclude file(s)
   */
  public function formatPublications($node, $files)
  {
    $bibs = $this->getPublications($files);

    if (!count($bibs)) {
      print_line( 'This page does not have a bibliography (yet)' );
      return false;
    }
    //print_line( $node->getAttribute('title') );
    $this->formatHeading($node, 1);
    print_line( '<ol id="publications">' );
    $this->formatBibFiles($bibs, "li");
    print_line( '</ol>' );
  }

  /**
   * Applies format to list items (itemize or enumerate)
   * @param $node  DOMElement  XML node
   */
  public function formatList($node)
  {
    $items = $this->getEntries($node);
    foreach ($items as $item) {
      print_line( '<li>' . $item . '</li>' );
    }
  }

  /**
   * Gets the (text) items in a node.
   * @param $node  DOMElement  XML node
   * @param $regex string      regular expression to split items (default: by newlines)
   * @return array
   */
  protected function getEntries($node)
  {
    $entries = preg_split($this->REGEX_ENTRIES, trim($node), 0, PREG_SPLIT_NO_EMPTY);
    $entries = array_map('trim', $entries);
    
    return $entries;
  }
  
}
?>
