<?php
require_once 'class.interface.mycvparser.php';
require_once 'class.mypublication.php';
require_once 'config.php';
require_once 'functions.php';
require_once 'functions.bib.php';

/**
 * A class for parsing CV pages.
 * @author Luis Leiva
 * @date 18 Jun 2010
 * @require class.interface.mycvparser.php
 * @require class.mypublication.php
 * @require functions.php
 * @require functions.bib.php
 */
class MyCVParser extends DOMDocument implements IMyCVParser
{
  /** Name of loaded XML file */
  public $filename;

  protected $cacheFile;
  protected $cacheTime;

  public function __construct($file)
  {
    $this->filename = $file;
    //$this->validateOnParse = true;

    $file_name = basename($this->filename, '.xml');
    $this->cacheFile = BASEPATH . '/cache-' . $file_name . '.html';
    $this->cacheTime = BASEPATH . '/cache-' . $file_name . '.time';

    $this->loadCache(); // If available.
    $this->load($file); // Otherwise, load DOM.
  }

  public function loadCache() {
    $cache_text = NULL;
    $cache_time = NULL;
    if (file_exists($this->cacheFile))
    	$cache_text = file_get_contents($this->cacheFile);
    if (file_exists($this->cacheTime))
    	$cache_time = file_get_contents($this->cacheTime);
    if (!empty($cache_text) && filemtime($this->filename) == $cache_time && !isset($_GET['nocache'])) {
      die($cache_text);
    }
  }

  public function saveCache($content) {
    file_put_contents($this->cacheFile, $content);
    file_put_contents($this->cacheTime, filemtime($this->filename));
  }
    
  // properties (used var instead of const to override'em in subclasses) --------
  
  /** Regular expression to extract node contents. */
  var $REGEX_ENTRIES = "/(\r?\n)+/"; // split by newlines
  
  /** Separator to get URL and caption from images. */
  var $SEPARATOR_TABLE       = "||";
  /** Separator to get columns from tables. */
  var $SEPARATOR_IMAGE       = "%%";
  /** Separator to get items from definition lists. */
  var $SEPARATOR_DESCRIPTION = "##";
  
  /** Prefix for table captions. */
  var $CAPTION_PREFIX_TABLE  = "Table:";
  /** Prefix for image captions. */
  var $CAPTION_PREFIX_IMAGE = "Figure:";
  
  // methods -------------------------------------------------------------------

  public function formatMetadata()
  {
    $meta = array();
    $data = $this->getMetadata();
    foreach ($data as $field => $value) {
      switch ($field) {
        case 'title':
          array_push($meta, 
                      '<title>'.$value.'</title>',
                      '<meta http-equiv="Title" content="'.$value.'" />',
                      '<meta name="Title" content="'.$value.'" />'
                    );
          break;
        case 'author':
            $meta[] = '<meta name="Author" content="'.$value.'" />';
          break;
        case 'description':
            $meta[] = '<meta name="Description" content="'.$value.'" />';
          break;
        case 'keywords':
            $meta[] = '<meta name="Keywords" content="'.$value.'" />';
          break;
        default: 
          break;
      }
    }
    $this->addDOMElement($meta, "head");
  }

  protected function getMetadata()
  {
    $fields = array();
    $metadata = $this->getElementsByTagName('metadata');
    foreach ($metadata as $meta)
    {
      $nodes = $meta->getElementsByTagName('*');
      foreach ($nodes as $node)
      {
        $fields[$node->nodeName] = $node->nodeValue;
      }
    }

    return $fields;
  }

  public function formatPersonal()
  {
    if ($this->getElementsByTagName('personal')->length > 0) {
      echo '<div id="personal">';
      $this->parsePersonal();
      echo '</div><!-- end personal -->';
    }
  }
  
  protected function parsePersonal()
  {
    $personal = $this->getElementsByTagName('personal');
    foreach ($personal as $mydata)
    {
      $nodes = $mydata->getElementsByTagName('*');
      foreach ($nodes as $node)
      {
        switch($node->nodeName)
        {
          case 'photo':
            $this->formatPhoto($node);
            break;
          case 'name':
            $this->formatName($node);
            break;
          case 'birthday':
            $this->formatBirthday($node);
            break;
          case 'address':
            $this->formatAddress($node);
            break;
          case 'phone':
            $this->formatPhone($node);
            break;
          case 'email':
            $this->formatEmail($node);
            break;
          case 'webpage':
            $this->formatWebpage($node);
            break;
          case 'occupation':
          case 'affiliation':
          case 'notes':
            $this->formatText($node);
            break;
          default:
            break;
        }
      }
    }
  }

  public function getPhoto(DOMElement $node)
  {
    $img = $node->getAttribute('uri');
    $alt = $node->getAttribute('alt');
    if (!empty($img)) {
      if (empty($alt)) $alt = "MyCV Photo";
      return '<img src="'.$img.'" alt="'.$alt.'" />';
    }
    
    return false;
  }
  
  public function formatPhoto(DOMElement $node)
  {
    $photo  = '<div class="personal-photo">';
    $photo .= $this->getPhoto($node);
    $photo .= '</div><!-- end personal-photo -->';
    echo $photo;
  }

  public function getName(DOMElement $node)
  {
    $entry = "";
    /*foreach ($node->attributes as $attrName => $attrNode) {
      if (!empty($attrNode->nodeValue)) {
        $entry .= " " . $attrNode->nodeValue;
      }
    }*/
    $salutation = $node->getAttribute('salutation');
    if (!empty($salutation)) {
      $entry .= $salutation . ". ";
    }    
    $entry .= $node->getAttribute('first') . " ";
    $middle = $node->getAttribute('middle');
    if (!empty($middle)) {
      $entry .= $middle . ". ";
    }
    $entry .= $node->getAttribute('last');
    
    return trim($entry);
  }
  
  public function formatName(DOMElement $node)
  {
    $name  = '<div class="personal-name">';
    $name .= $this->getName($node);
    $name .= '</div><!-- end personal-name -->';
    echo $name;
  }

  public function getBirthday(DOMElement $node)
  {
    if (!$node->hasAttribute('hidden')) {
      $display = true;
    } else {
      $hidden = $node->getAttribute('hidden');
      $display = ($hidden == "false" || $hidden == "0");
    }

    if ($display) {
      // check typed month
      $month = $node->getAttribute('month');
      if (strlen($month) < 3) {
        $month = date('F', mktime(0,0,0,$month,1));
      }
      return trim( $node->getAttribute('day') ." ". $month ." ". $node->getAttribute('year') );
    }
  }
  
  public function formatBirthday(DOMElement $node)
  {
    $bday  = '<div class="personal-birthday">';
    $bday .= $this->getBirthday($node);
    $bday .= '</div><!-- end personal-birthday -->';
    echo $bday;
  }

  public function getAddress(DOMElement $node)
  {
    $entry  = $node->getAttribute('street');
    $entry .= " " . $node->getAttribute('zip');
    $entry .= " &mdash; ";
    $entry .= $node->getAttribute('city');
    $entry .= " (" . $node->getAttribute('country') . ")";

    return $entry;
  }
  
  public function formatAddress(DOMElement $node)
  {
    $address  = '<div class="personal-address">';
    $address .= $this->getAddress($node);
    $address .= '</div><!-- end personal-address -->';
    echo $address;
  }

  public function getPhone(DOMElement $node)
  {
    $entry  = $node->getAttribute('prefix');
    $entry .= " " . $node->getAttribute('number');
    $extension = $node->getAttribute('extension');
    if (!empty($extension)) {
      $entry .= " (ext. " . $extension . ")";
    }

    return $entry;
  }
  
  public function formatPhone(DOMElement $node)
  {
    $phone  = '<div class="personal-phone">';
    $phone .= $this->getPhone($node);
    $phone .= '</div><!-- end personal-phone -->';
    echo $phone;
  }

  public function getEmail(DOMElement $node)
  {
    $mail = $node->getAttribute('inbox');
    // protect email from spammers
    $mail = str_replace("@", " AT_SYMBOL ", $mail);
    $mail = str_replace(".", " DOT ", $mail);
    
    return $mail;
  }
  
  public function formatEmail(DOMElement $node)
  {
    $email  = '<div class="personal-email">';
    $email .= $this->getEmail($node);
    $email .= '</div><!-- end personal-email -->';
    echo $email;
  }

  public function getWebpage(DOMElement $node)
  {
    $uri = url_check_protocol( $node->getAttribute('uri') );
    
    $entry = "";
    if( $node->hasAttribute('desc') ) {
      $entry .= '<span>' . $node->getAttribute('desc') . '</span>';
    }
    $entry .= ' <a href="'.$uri.'">' . $uri . '</a>';

    return $entry;
  }
  
  public function formatWebpage(DOMElement $node)
  {
    $webpage  = '<div class="personal-webpage">';
    $webpage .= $this->getWebpage($node);
    $webpage .= '</div><!-- end personal-webpage -->';
    echo $webpage;
  }

  public function getText(DOMElement $node)
  {
    $entry = "";
    $lines = $this->getEntries($node);
    foreach ($lines as $line) {
      $entry .= '<p>' . $line . '</p>';
    }

    return $entry;
  }
  
  public function formatText(DOMElement $node)
  {
    $text  = '<div class="personal-text">';
    $text .= $this->getText($node);
    $text .='</div><!-- end personal-text -->';
    echo $text;
  }
  
  public function formatSectionGroups()
  {
    echo '<div id="sections">';
    $this->parseSectionGroups();
    echo '</div><!-- end sections -->';
  }
  
  protected function parseSectionGroups()
  {
    $sectiongroups = $this->getElementsByTagName('sectiongroup');
    foreach ($sectiongroups as $sectiongroup)
    {
      echo '<fieldset class="section-group">';
      echo '<legend>' . $sectiongroup->getAttribute("title") . '</legend>';
      $contents = $sectiongroup->getElementsByTagName('*');
      foreach ($contents as $sectionindex => $node)
      {
        $name = $node->nodeName;
        switch ($name)
        {
          case 'section':
          default:
            $level = (int)$node->getAttribute('level');
            $this->formatHeading($node, $level);
            break;

          case 'publications':
            $files = array();
            if( $node->getAttribute('include') ) {
              $files['include'] = csv2array($node->getAttribute('include'));
            } else if( $node->getAttribute('exclude') ) {
              $files['exclude'] = csv2array($node->getAttribute('exclude'));
            }
            $this->formatPublications($node, $files);
            break;

          case 'text':
            // just display raw node value
            echo '<div class="section-text">';
            //echo nl2br(trim($node->nodeValue));
	          echo trim($node->nodeValue);
            echo '</div><!-- end section-text -->';
            break;

          case 'env':
            // apply some formatting
            $type = $node->getAttribute('type');
            switch ($type)
            {
              case 'tabular':
                $this->formatTable($node);
                break;
              case 'itemize':
                $this->formatItemize($node);
                break;
              case 'enumerate':
                $this->formatEnumerate($node);
                break;
              case 'description':
                $this->formatDescription($node);
                break;
              case 'image':
                $this->formatImage($node);
              default:
                break;
            }
            break;
        }
      }
          
      echo '</fieldset><!-- end section-group -->';
    }
  }

  /**
   * Applies format to image items
   * @param $node  DOMElement  XML node
   * @param $level int         Heading level (1, 2, 3, or 4)
   */
  public function formatHeading(DOMElement $node, $level)
  {
    if ($level < 1) $level = 1;
    if ($level > 6) $level = 6;

    $tag = 'h'.$level;
    $title = $node->getAttribute('title');
    echo '<'.$tag.' class="section-heading">' . $title . '</'.$tag.'>';
  }

  /**
   * Applies format to tabular items
   * @param $node  DOMElement  XML node
   */
  public function formatTable(DOMElement $node)
  {
    $hasCaption = false; // flag to better parse the table
    
    $table = '<table class="environment env-table">';
    $rows = $this->getEntries($node);
    foreach ($rows as $index => $row)
    {
      if ($index == 0 && strpos($row, $this->SEPARATOR_TABLE) === false)
      {
        $hasCaption = true;
        $entry = "";
        // check if caption was typed
        if ( (bool)$this->CAPTION_PREFIX_TABLE) {
          $entry .= '<caption><span>' . $this->CAPTION_PREFIX_TABLE .'</span> '. $row . '</caption>';
        } else /*if (!empty($row))*/ {
          $entry .= '<caption>' . $row . '</caption>';
        }
        $table .= $entry;
        continue;
      }
      
      $cols = explode($this->SEPARATOR_TABLE, $row);
      // first row are table headings
      if( ($hasCaption && $index == 1) || (!$hasCaption && $index == 0) ) {
        $table .= '<thead>';
        $table .= '<tr>';
        foreach ($cols as $col) {
         $table .= '<th>' . trim($col) . '</th>';
        }
        $table .='</tr>';
        $table .='</thead>';
        continue;
      }
      // the rest are regular rows
      $table .='<tbody>';
      $table .='<tr>';
      foreach ($cols as $col) {
        $table .= '<td>' . trim($col) . '</td>';
      }
      $table .='</tr>';
      $table .='</tbody>';
    }
    $table .='</table>';
    echo $table;
  }

  /**
   * Applies format to itemize lists
   * @param $node  DOMElement  XML node
   */
  public function formatItemize(DOMElement $node)
  {
    echo '<ul class="environment env-itemize">';
    $this->formatList($node);
    echo '</ul>';
  }

  /**
   * Applies format to enumerate lists
   * @param $node  DOMElement  XML node
   */
  public function formatEnumerate(DOMElement $node)
  {
    echo '<ol class="environment env-enumerate">';
    $this->formatList($node);
    echo '</ol>';
  }

  /**
   * Applies format to description lists
   * @param $node  DOMElement  XML node
   */
  public function formatDescription(DOMElement $node)
  {
    $description = '<ul class="environment env-description">';
    $items = $this->getEntries($node);
    foreach ($items as $item) {
      list($dt,$dd) = explode($this->SEPARATOR_DESCRIPTION, $item);
      $description .= '<li><span>' . trim($dt) . '</span> ' . trim($dd) . '</li>';
    }
    $description .= '</ul>';
    echo $description;
  }

  /**
   * Applies format to enumerate lists
   * @param $node  DOMElement  XML node
   */
  public function formatImage(DOMElement $node)
  {
    $items = $this->getEntries($node);
    // check if global caption should be typed
    if (count($items) > 2)
    {
      $globalcaption = "";
      if (strpos($items[0], $this->SEPARATOR_IMAGE) === false) {
        $globalcaption .= '<p class="caption-all">';
        if ( (bool)$this->CAPTION_PREFIX_IMAGE) {
          $globalcaption .= '<span>'.$this->CAPTION_PREFIX_IMAGE.'</span> ';
        }
        $globalcaption .= $items[0] . '</p>';
      }
      echo '<div class="environment env-image-wrap">';
    }
    // now display images
    foreach ($items as $index => $item)
    {
      // check for errors
      if( (count($items) > 2 && $index == 0) /*|| strpos($item, $this->SEPARATOR_IMAGE) === false*/ ) continue;
      // get image parts
      list($uri,$caption) = explode($this->SEPARATOR_IMAGE, $item);
      // check for dot
      $dotpos = strpos($item, ".");
      if ($dotpos === false || $dotpos == strlen($item) - 1) continue;
      
      echo '<div class="environment env-image">';
      echo '<img src="'.$uri.'" alt="Image" />';
      if (!empty($caption))
      {
        $entry  = '<p class="caption">';
        if (count($items) > 2) {
          $entry .= '<span>('.$index.')</span> ';
        } else {
          if ( (bool)$this->CAPTION_PREFIX_IMAGE) {
            $entry .= '<span>'.$this->CAPTION_PREFIX_IMAGE.'</span> ';
          }
        }
        $entry .= $caption . '</p>';

        echo $entry;
      }
      echo '</div><!-- end env-image -->';
    }
    
    if (isset($globalcaption)) {
      echo $globalcaption;
      echo '</div><!-- end env-image-wrap -->';
    }
  }
  
  /**
   * Retrieves all BIB files from bib/ dir.
   * @param $files  array optional list of files to load
   */
  protected function loadPublicationFiles($files)
  {
    $bibs = array();
  	$dir = realpath(SYSPATH.'../bib/');
    if ($handle = opendir($dir)) {
      while (false !== ($file = readdir($handle))) {
        $ext = file_info($file, "extension");
        if ($ext == "bib")
        {
          $name = strtolower( file_info($file, "filename") );
          if (array_key_exists('include', $files) && count($files['include']) > 0) {
            // user want to *include* a set of BIB files
            if (!in_array($name, $files['include'])) continue;
          } else if (array_key_exists('exclude', $files) && count($files['exclude']) > 0) {
            // user want to *exclude* a set of BIB files (watch out in_array condition)
            if (in_array($name, $files['exclude'])) continue;
          }
          $bibs[] = $dir.'/'.$file;
        }
      }
      closedir($handle);
    }

    return $bibs;
  }

  /**
   * Parse .BIB files
   * @param $bibs  array list of files to parse
   */
  protected function parseBibFiles($bibs)
  {
    foreach ($bibs as $bibfile) {
      $parse = new PARSEENTRIES();
      //$parse->expandMacro = true;
      $parse->openBib($bibfile);
      $parse->extractEntries();
      $parse->closeBib();
      list($preamble, $strings, $entries, $undefinedStrings) = $parse->returnArrays();

      // by now we only need the entries
      return $entries;
    }
  }

  /**
   * Formats retrieved BIB files
   * @param $entries array list of parsed BIBs
   * @param $wrapitem string HTML element wrapper for each item (e.g., "div")
   */
  public function formatBibFiles($entries, $wrapitem)
  {
    foreach ($entries as $entry) {
      echo '<'.$wrapitem.' class="bib-entry '.$entry['bibtexEntryType'].'">';
      $parsedbib = new MyPublication($entry);
      //MyPublication->delatex($text)
      echo '</'.$wrapitem.'><!-- end bib-entry -->';
    }
  }


  /**
   * Applies format to publication items
   * @param $node  DOMElement  XML node
   * @param $files array  include or exclude file(s)
   */
  public function formatPublications(DOMElement $node, $files)
  {
    // load publications files
    $bibs = $this->loadPublicationFiles($files);
    if (!count($bibs)) {
      echo 'Bibliography not found.';
      return false;
    }
    // optionally sort them
    $entries = $this->getPublications($node, $bibs);
    // display (use class instead of ID, as there may be more than one 'publications' node)
    $numbering = $node->getAttribute('numbering');
    $numberingtype = $node->getAttribute('numberingtype');
    echo '<ol class="publications"';
    if ($numbering) echo ' ' . $numbering;
    if ($numberingtype) echo ' type="' . $numberingtype .'"';
    echo '>';
    $this->formatBibFiles($entries, "li");
    echo '</ol>';
  }

  /**
   * Retrieves parsed BIB files and optionally sort them.
   * @param $node DOMElement XML node
   * @param $bibs array loaded publications
   */
  protected function getPublications(DOMElement $node, $bibs)
  {
    $entries = $this->parseBibFiles($bibs); 
    $sortby = $node->getAttribute('sortby');
    switch($sortby) {
      case 'author':
        $key = 'author';
        break;
      case 'title':
        $key = 'title';
        break;
      case 'year':
        $key = 'year';
        break;
      case 'month': // TODO
        $key = 'month';
        break;
      case 'type':
        $key = 'bibtexEntryType';
        break;
      case 'citation':
        $key = 'bibtexCitation';
        break;
      default:
        $key = NULL;
        break;
    }
    $order = $node->getAttribute('order');
    if (empty($order)) $order = "asc";
    // set ordering
    if ($order == "asc") {
      $a = 'a';
      $b = 'b';
    } else {
      $a = 'b';
      $b = 'a';
    }
    if ($key) {
      #$fn  = 'if ($'.$a.'["'.$key.'"] == $'.$b.'["'.$key.'"]) return 0;';
      #$fn .= 'return $'.$a.'["'.$key.'"] < $'.$b.'["'.$key.'"] ? -1 : 1;';                                
      #$cmp = create_function('$a,$b', $fn);
      # NEW: In PHP 8 we use build_sorter() declared in functions.php
      usort($entries, build_sorter($key));
    }
    
    return $entries;
  }

  /**
   * Applies format to list items (itemize or enumerate)
   * @param $node  DOMElement  XML node
   */
  public function formatList(DOMElement $node)
  {
    $items = $this->getEntries($node);
    foreach ($items as $item) {
      echo '<li>' . $item . '</li>';
    }
  }

  /**
   * Gets the (text) items in a node.
   * @param $node  DOMElement  XML node
   * @param $regex string      regular expression to split items (default: by newlines)
   * @return array
   */
  protected function getEntries(DOMElement $node)
  {
    $entries = preg_split($this->REGEX_ENTRIES, trim($node->nodeValue), 0, PREG_SPLIT_NO_EMPTY);
    $entries = array_map('trim', $entries);
    
    return $entries;
  }
  
}
?>
