<?php
require_once 'functions.php';
require_once 'functions.bib.php';

/**
 * A class for parsing BibTeX files. Lite version.
 * The constructor accepts an array generated with the bibtexParse package.
 * @see http://bibliophile.sourceforge.net
 * @author Luis Leiva
 * @date 18 Jun 2010
 * @require functions.php
 * @require functions.bib.php
 */
class MyPublication
{
  /** Maximum number of BIB authors. If exceed, puts 'et al.'. */
  var $maxNumAuthors = 3;

  // these should be constants, but one could change tokens in subclasses

  /** Punctuation dot. */
  var $DOT    = ". ";
  /** Punctuation comma. */
  var $COMMA  = ", ";
  /** Punctuation colon. */
  var $COLON  = "; ";

  /**
   * Constructor.
   * @param $e array fields
   */  
  public function MyPublication($e)
  {
    switch($e['bibtexEntryType'])
    {
      case 'article':
        $this->parseArticle($e);
        break;
      case 'book':
        $this->parseBook($e);
        break;
      case 'booklet':
        $this->parseBooklet($e);
        break;
      case 'conference':
      case 'inproceedings':
        $this->parseInProceedings($e);
        break;
      case 'inbook':
        $this->parseInBook($e);
        break;
      case 'incollection':
        $this->parseInCollection($e);
        break;
      case 'manual':
        $this->parseManual($e);
        break;
      case 'masterthesis':
        $this->parseMasterThesis($e);
        break;
      case 'misc':
        $this->parseMisc($e);
        break;
      case 'phdthesis':
        $this->parsePhDThesis($e);
        break;
      case 'proceedings':
        $this->parseProceedings($e);
        break;
      case 'techreport':
        $this->parseTechReport($e);
        break;
      case 'unpublished':
        $this->parseUnPublished($e);
        break;
      default:
        break;
    }
  }

  /**
   * Parses BibTeX article:
   * An article from a journal or magazine.
   * Required fields: author, title, journal, year
   * Optional fields: volume, number, pages, month, note, key
   * @param $e array fields
   */
  protected function parseArticle($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryJournal($e);
    //$this->formatEntryYear($e);
    
    $this->formatEntryVolume($e);
    $this->formatEntryNumber($e);
    $this->formatEntryPages($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX book:
   * A book with an explicit publisher.
   * Required fields: author/editor, title, publisher, year
   * Optional fields: volume, series, address, edition, month, note, key
   * @param $e array fields
   */
  protected function parseBook($e)
  {
    $this->formatEntryAuthor($e); //$this->formatEntryEditor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryPublisher($e);
    //$this->formatEntryYear($e);

    $this->formatEntryVolume($e);
    $this->formatEntrySeries($e);
    $this->formatEntryAddress($e);
    $this->formatEntryEdition($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX booklet:
   * A work that is printed and bound, but without a named publisher or sponsoring institution.
   * Required fields: title
   * Optional fields: author, howpublished, address, month, year, note, key
   * @param $e array fields
   */
  protected function parseBooklet($e)
  {
    $this->formatEntryTitle($e);
    
    $this->formatEntryAuthor($e);
    $this->formatEntryHowpublished($e);
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e);
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX inproceedings:
   * An article in a conference proceedings.
   * BibTeX conference is an alias of BibTeX inproceedings, included for Scribe compatibility.
   * Required fields: author, title, booktitle, year
   * Optional fields: editor, pages, organization, publisher, address, month, note, key
   * @param $e array fields
   */
  protected function parseInProceedings($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryBooktitle($e);
    //$this->formatEntryYear($e);

    $this->formatEntryEditor($e);
    $this->formatEntryPages($e);
    $this->formatEntryOrganization($e);
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX inbook:
   * A part of a book, usually untitled. May be a chapter (or section or whatever) and/or a range of pages.
   * Required fields: author/editor, title, chapter/pages, publisher, year
   * Optional fields: volume, series, address, edition, month, note, key
   * @param $e array fields
   */
  protected function parseInBook($e)
  {
    $this->formatEntryAuthor($e); //$this->formatEntryEditor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryChapter($e); //$this->formatEntryPages($e);
    $this->formatEntryPublisher($e);
    //$this->formatEntryYear($e);

    $this->formatEntryVolume($e);
    $this->formatEntrySeries($e);
    $this->formatEntryAddress($e);
    $this->formatEntryEdition($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX incollection:
   * A part of a book having its own title.
   * Required fields: author, title, booktitle, year
   * Optional fields: editor, pages, organization, publisher, address, month, note, key
   * @param $e array fields
   */
  protected function parseInCollection($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryBooktitle($e);
    //$this->formatEntryYear($e);

    $this->formatEntryEditor($e);
    $this->formatEntryPages($e);
    $this->formatEntryOrganization($e);
    $this->formatEntryPublisher($e);
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX manual:
   * Technical documentation.
   * Required fields: title
   * Optional fields: author, organization, address, edition, month, year, note, key
   * @param $e array fields
   */
  protected function parseManual($e)
  {
    $this->formatEntryTitle($e);
    
    $this->formatEntryAuthor($e);
    $this->formatEntryOrganization($e);
    $this->formatEntryAddress($e);
    $this->formatEntryEdition($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e);
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX mastersthesis:
   * A Master's thesis.
   * Required fields: author, title, school, year
   * Optional fields: address, month, note, key
   * @param $e array fields
   */
  protected function parseMasterThesis($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntrySchool($e);
    //$this->formatEntryYear($e);
    
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX misc:
   * For use when nothing else fits.
   * Required fields: none
   * Optional fields: author, title, howpublished, month, year, note, key
   * @param $e array fields
   */
  protected function parseMisc($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryHowpublished($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e);
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX phdthesis:
   * A Ph.D. thesis.
   * Required fields: author, title, school, year
   * Optional fields: address, month, note, key
   * @param $e array fields
   */
  protected function parsePhDThesis($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntrySchool($e);
    //$this->formatEntryYear($e);
    
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX proceedings:
   * The proceedings of a conference.
   * Required fields: title, year
   * Optional fields: editor, publisher, organization, address, month, note, key
   * @param $e array fields
   */
  protected function parseProceedings($e)
  {
    $this->formatEntryTitle($e);
    //$this->formatEntryYear($e);

    $this->formatEntryEditor($e);
    $this->formatEntryPublisher($e);
    $this->formatEntryOrganization($e);
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX techreport:
   * A report published by a school or other institution, usually numbered within a series.
   * Required fields: author, title, institution, year
   * Optional fields: type, number, address, month, note, key
   * @param $e array fields
   */
  protected function parseTechReport($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryInstitution($e);
    //$this->formatEntryYear($e);

    $this->formatEntryType($e);
    $this->formatEntryNumber($e);
    $this->formatEntryAddress($e);
    $this->formatEntryMonth($e);
    $this->formatEntryNote($e);
    $this->formatEntryYear($e); // year better here
    $this->formatEntryKey($e);
  }
  
  /**
   * Parses BibTeX unpublished:
   * A document having an author and title, but not formally published.
   * Required fields: author, title, note
   * Optional fields: month, year, key
   * @param $e array fields
   */
  protected function parseUnPublished($e)
  {
    $this->formatEntryAuthor($e);
    $this->formatEntryTitle($e);
    $this->formatEntryNote($e);
    
    $this->formatEntryMonth($e);
    $this->formatEntryYear($e);
    $this->formatEntryKey($e);
  }

  /**
   * Applies HTML format to BIB author(s): <span class="bib-author">
   * @param $e array fields
   */
  public function formatEntryAuthor($e)
  {
    if (!$this->isValidArrayKey($e, 'author')) return;

    // authors can be divided by "and" or "&"
    $authors = preg_split("/\s(and|&)\s/i", $e['author']);
    if (count($authors) > $this->maxNumAuthors) {
      $entry = "";
      foreach ($authors as $index => $name) {
        if ($index == $this->maxNumAuthors) break;
        $entry .= $name . ", ";
      }
      $entry .= "et al"; // do not put the final dot, since it is added at last
    } else {
      $entry = implode(", ", $authors);
    }
    
    print_line( ' <span class="bib-author">' . $this->delatex($entry) . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB title: <span class="bib-title">
   * @param $e array fields
   */  
  public function formatEntryTitle($e)
  {
    if (!$this->isValidArrayKey($e, 'title')) return;

    // convert LaTeX dashes to HTML entities
    $title = preg_replace('#[--]+[-]#', "&ndash;", $e['title']);

    print_line( ' <span class="bib-title">' . $this->delatex($title) . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB journal: <span class="bib-journal">
   * @param $e array fields
   */    
  public function formatEntryJournal($e)
  {
    if (!$this->isValidArrayKey($e, 'journal')) return;
    
    print_line( ' <span class="bib-journal">' . $e['journal'] . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB year: <span class="bib-year">
   * @param $e array fields
   */    
  public function formatEntryYear($e)
  {
    if (!$this->isValidArrayKey($e, 'year')) return;
    
    print_line( ' <span class="bib-year">' . $e['year'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB volume: <span class="bib-volume">
   * @param $e array fields
   */   
  public function formatEntryVolume($e)
  {
    if (!$this->isValidArrayKey($e, 'volume')) return;
    
    print_line( ' Vol. <span class="bib-volume">' . $e['volume'] . '</span>' );
  }

  /**
   * Applies HTML format to BIB number: <span class="bib-number">
   * @param $e array fields
   */   
  public function formatEntryNumber($e)
  {
    if (!$this->isValidArrayKey($e, 'number')) return;
    
    print_line( '(<span class="bib-number">' . $e['number'] . '</span>)' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB series: <span class="bib-series">
   * @param $e array fields
   */   
  public function formatEntrySeries($e)
  {
    if (!$this->isValidArrayKey($e, 'series')) return;
    
    print_line( ' <span class="bib-series">' . $e['series'] . '</span>' . $this->COMMA );
  }
  
  /**
   * Applies HTML format to BIB booktitle: <span class="bib-booktitle">
   * @param $e array fields
   */   
  public function formatEntryBooktitle($e)
  {
    if (!$this->isValidArrayKey($e, 'booktitle')) return;
    
    print_line( " In " . '<span class="bib-booktitle">' . $e['booktitle'] . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB editor: <span class="bib-editor">
   * @param $e array fields
   */     
  public function formatEntryEditor($e)
  {
    if (!$this->isValidArrayKey($e, 'editor')) return;
    
    print_line( ' <span class="bib-editor">' . $e['editor'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB pages: <span class="bib-pages">
   * @param $e array fields
   */       
  public function formatEntryPages($e)
  {
    if (!$this->isValidArrayKey($e, 'pages')) return;

    $parsePage = new PARSEPAGE();
    list($start, $end) = $parsePage->init($e['pages']);
    $entry = $end ? "pp. " . $start . "&ndash;" . $end : "p. " . $start;

    print_line( ' <span class="bib-pages">' . $entry . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB organization: <span class="bib-organization">
   * @param $e array fields
   */  
  public function formatEntryOrganization($e)
  {
    if (!$this->isValidArrayKey($e, 'organization')) return;
    
    print_line( ' <span class="bib-organization">' . $e['organization'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB chapter: <span class="bib-chapter">
   * @param $e array fields
   */    
  public function formatEntryChapter($e)
  {
    if (!$this->isValidArrayKey($e, 'chapter')) return;
    
    print_line( ' <span class="bib-chapter"> Chp. ' . $e['chapter'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB publisher: <span class="bib-publisher">
   * @param $e array fields
   */    
  public function formatEntryPublisher($e)
  {
    if (!$this->isValidArrayKey($e, 'publisher')) return;
    
    print_line( ' <span class="bib-publisher">' . $e['publisher'] . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB edition: <span class="bib-edition">
   * @param $e array fields
   */    
  public function formatEntryEdition($e)
  {
    if (!$this->isValidArrayKey($e, 'edition')) return;
    
    print_line( ' <span class="bib-edition">' . $e['edition'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB school: <span class="bib-school">
   * @param $e array fields
   */    
  public function formatEntrySchool($e)
  {
    if (!$this->isValidArrayKey($e, 'school')) return;
    
    print_line( ' <span class="bib-school">' . $e['school'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB howpublished: <span class="bib-howpublished">
   * @param $e array fields
   */   
  public function formatEntryHowpublished($e)
  {
    if (!$this->isValidArrayKey($e, 'howpublished')) return;
    
    str_link_urls($e['howpublished']);
    print_line( ' <span class="bib-howpublished">' . $e['howpublished'] . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB institution: <span class="bib-institution">
   * @param $e array fields
   */     
  public function formatEntryInstitution($e)
  {
    if (!$this->isValidArrayKey($e, 'institution')) return;
    
    print_line( ' <span class="bib-institution">' . $e['institution'] . '</span>' . $this->DOT );
  } 

  /**
   * Applies HTML format to BIB type: <span class="bib-type">
   * @param $e array fields
   */    
  public function formatEntryType($e)
  {
    if (!$this->isValidArrayKey($e, 'type')) return;
    
    print_line( ' <span class="bib-type">' . $e['type'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB address: <span class="bib-address">
   * @param $e array fields
   */     
  public function formatEntryAddress($e)
  {
    if (!$this->isValidArrayKey($e, 'address')) return;
    
    print_line( ' <span class="bib-address">' . $e['address'] . '</span>' . $this->COMMA );
  }

  /**
   * Applies HTML format to BIB month: <span class="bib-month">
   * @param $e array fields
   */  
  public function formatEntryMonth($e)
  {
    if (!$this->isValidArrayKey($e, 'month')) return;
    
    print_line( ' <span class="bib-month">' . $e['month'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB note: <span class="bib-note">
   * @param $e array fields
   */  
  public function formatEntryNote($e)
  {
    if (!$this->isValidArrayKey($e, 'note')) return;
    
    str_link_urls($e['note']);
    print_line( ' <span class="bib-note">' . $e['note'] . '</span>' . $this->DOT );
  }

  /**
   * Applies HTML format to BIB key: <span class="bib-key">
   * @param $e array fields
   */  
  public function formatEntryKey($e)
  {
    if (!$this->isValidArrayKey($e, 'key')) return;
    
    print_line( ' <span class="bib-key">' . $e['key'] . '</span>' . $this->DOT );
  }

  /**
   * Ensures that a key exists in array, checking also that it's not empty.
   * @param $e array fields
   * @param $key string array key
   */
  protected function isValidArrayKey($e, $key)
  {
    if (array_key_exists($key, $e)) {
      $test = trim($e[$key]);
      return !empty($test);
    } else {
      return false;
    }
  }

  /**
   * Converts LaTeX dashes to HTML entities.
   * @param $text string
   */  
  protected function fixnumrange($text)
  {
    return preg_replace('#[--]+[-]#', "&ndash;", $text);
  }

  /**
   * Clears LaTeX chars. This method is not yet complete -- ToDo: parse LaTeX special chars, accents, etc.
   * @param $text string
   */
  protected function delatex($text)
  {
    $patterns = array(
                       '#\\\.#',    // escaped entities
                       '#\{?\}?#',  // brackets
                       '#~#'        // spaces
                     );
    $text = preg_replace($patterns, "", $text);

    return $text;
  }

}
?>
