<?php
require_once 'class.interface.mypublication.php';
require_once 'functions.php';
require_once 'functions.bib.php';

/**
 * A class for parsing BibTeX files.
 * The constructor accepts an array generated with the bibtexParse package.
 * @see http://bibliophile.sourceforge.net
 * @author Luis Leiva
 * @date 18 Jun 2010
 * @require class.interface.mypublication.php
 * @require functions.php
 * @require functions.bib.php
 */
class MyPublication implements IMyPublication
{
  /** Maximum number of BIB authors. If exceed, puts 'et al.'. */
  var $maxNumAuthors = 5;
  // these should be constants, but one could change the tokens in subclasses

  /** Punctuation dot. */
  var $DOT    = ". ";
  /** Punctuation comma. */
  var $COMMA  = ", ";
  /** Punctuation colon. */
  var $COLON  = "; ";

  /**
   * Constructor.
   * @param $e array fields
   */    
  public function __construct(array $e)
  {
    switch($e['bibtexEntryType'])
    {
      case 'article':
        $this->parseArticle($e);
        break;
      case 'book':
        $this->parseBook($e);
        break;
      case 'booklet':
        $this->parseBooklet($e);
        break;
      case 'conference':
      case 'inproceedings':
        $this->parseInProceedings($e);
        break;
      case 'inbook':
        $this->parseInBook($e);
        break;
      case 'incollection':
        $this->parseInCollection($e);
        break;
      case 'manual':
        $this->parseManual($e);
        break;
      case 'masterthesis':
        $this->parseMasterThesis($e);
        break;
      case 'misc':
        $this->parseMisc($e);
        break;
      case 'phdthesis':
        $this->parsePhDThesis($e);
        break;
      case 'proceedings':
        $this->parseProceedings($e);
        break;
      case 'techreport':
        $this->parseTechReport($e);
        break;
      case 'unpublished':
        $this->parseUnPublished($e);
        break;
      default:
        break;
    }
  }

  /**
   * Parses BibTeX article:
   * An article from a journal or magazine.
   * Required fields: author, title, journal, year
   * Optional fields: volume, number, pages, month, note, key
   * @param $e array fields
   */
  protected function parseArticle(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryJournal($e);
    //$entry .= $this->formatEntryYear($e);
    
    $entry .= $this->formatEntryVolume($e);
    $entry .= $this->formatEntryNumber($e);
    $entry .= $this->formatEntryPages($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here    
    $entry .= $this->formatEntryNote($e);
    $entry .= $this->formatEntryKey($e);

    echo '<span class="article">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX book:
   * A book with an explicit publisher.
   * Required fields: author/editor, title, publisher, year
   * Optional fields: volume, series, address, edition, month, note, key
   * @param $e array fields
   */
  protected function parseBook(array $e)
  {
    $entry  = $this->formatEntryAuthor($e); //$this->formatEntryEditor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryPublisher($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryVolume($e);
    $entry .= $this->formatEntrySeries($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryEdition($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);
    $entry .= $this->formatEntryKey($e);

    echo '<span class="book">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX booklet:
   * A work that is printed and bound, but without a named publisher or sponsoring institution.
   * Required fields: title
   * Optional fields: author, howpublished, address, month, year, note, key
   * @param $e array fields
   */
  protected function parseBooklet(array $e)
  {
    $entry  = $this->formatEntryTitle($e);
    
    $entry .= $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryHowpublished($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e);
    $entry .= $this->formatEntryNote($e);
    $entry .= $this->formatEntryKey($e);

    echo '<span class="booklet">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX inproceedings:
   * An article in a conference proceedings.
   * BibTeX conference is an alias of BibTeX inproceedings, included for Scribe compatibility.
   * Required fields: author, title, booktitle, year
   * Optional fields: editor, pages, organization, publisher, address, month, note, key
   * @param $e array fields
   */
  protected function parseInProceedings(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryBooktitle($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryEditor($e);
    $entry .= $this->formatEntryPages($e);
    $entry .= $this->formatEntryOrganization($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="inproceedings">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX inbook:
   * A part of a book, usually untitled. May be a chapter (or section or whatever) and/or a range of pages.
   * Required fields: author/editor, title, chapter/pages, publisher, year
   * Optional fields: volume, series, address, edition, month, note, key
   * @param $e array fields
   */
  protected function parseInBook(array $e)
  {
    $entry = "";
    if (isset($e['author'])) {
      $entry .= $this->formatEntryAuthor($e);
    }
    $entry .= $this->formatEntryChapter($e);
    $entry .= $this->formatEntryTitle($e);
    if (isset($e['editor'])) {
      $entry .= $this->formatEntryEditor($e);
    }
    $entry .= $this->formatEntryPublisher($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryVolume($e);
    $entry .= $this->formatEntrySeries($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryEdition($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="inbook">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX incollection:
   * A part of a book having its own title.
   * Required fields: author, title, booktitle, year
   * Optional fields: editor, pages, organization, publisher, address, month, note, key
   * @param $e array fields
   */
  protected function parseInCollection(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryBooktitle($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryEditor($e);
    $entry .= $this->formatEntryPages($e);
    $entry .= $this->formatEntryOrganization($e);
    $entry .= $this->formatEntryPublisher($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="incollection">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX manual:
   * Technical documentation.
   * Required fields: title
   * Optional fields: author, organization, address, edition, month, year, note, key
   * @param $e array fields
   */
  protected function parseManual(array $e)
  {
    $entry  = $this->formatEntryTitle($e);
    
    $entry .= $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryOrganization($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryEdition($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e);
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="manual">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX mastersthesis:
   * A Master's thesis.
   * Required fields: author, title, school, year
   * Optional fields: address, month, note, key
   * @param $e array fields
   */
  protected function parseMasterThesis(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntrySchool($e);
    //$entry .= $this->formatEntryYear($e);
    
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="masterthesis">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX misc:
   * For use when nothing else fits.
   * Required fields: none
   * Optional fields: author, title, howpublished, month, year, note, key
   * @param $e array fields
   */
  protected function parseMisc(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryHowpublished($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e);
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="misc">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX phdthesis:
   * A Ph.D. thesis.
   * Required fields: author, title, school, year
   * Optional fields: address, month, note, key
   * @param $e array fields
   */
  protected function parsePhDThesis(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntrySchool($e);
    //$entry .= $this->formatEntryYear($e);
    
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="phdthesis">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX proceedings:
   * The proceedings of a conference.
   * Required fields: title, year
   * Optional fields: editor, publisher, organization, address, month, note, key
   * @param $e array fields
   */
  protected function parseProceedings(array $e)
  {
    $entry  = $this->formatEntryTitle($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryEditor($e);
    $entry .= $this->formatEntryPublisher($e);
    $entry .= $this->formatEntryOrganization($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="proceedings">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX techreport:
   * A report published by a school or other institution, usually numbered within a series.
   * Required fields: author, title, institution, year
   * Optional fields: type, number, address, month, note, key
   * @param $e array fields
   */
  protected function parseTechReport(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryInstitution($e);
    //$entry .= $this->formatEntryYear($e);

    $entry .= $this->formatEntryType($e);
    $entry .= $this->formatEntryNumber($e);
    $entry .= $this->formatEntryAddress($e);
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e); // year better here
    $entry .= $this->formatEntryNote($e);    
    $entry .= $this->formatEntryKey($e);

    echo '<span class="techreport">'.$this->delatex($entry).'</span>';
  }
  
  /**
   * Parses BibTeX unpublished:
   * A document having an author and title, but not formally published.
   * Required fields: author, title, note
   * Optional fields: month, year, key
   * @param $e array fields
   */
  protected function parseUnPublished(array $e)
  {
    $entry  = $this->formatEntryAuthor($e);
    $entry .= $this->formatEntryTitle($e);
    $entry .= $this->formatEntryNote($e);
    
    $entry .= $this->formatEntryMonth($e);
    $entry .= $this->formatEntryYear($e);
    $entry .= $this->formatEntryKey($e);

    echo '<span class="unpublished">'.$this->delatex($entry).'</span>';
  }

  /**
   * Applies HTML format to BIB author(s): <span class="bib-author">
   * @param $e array fields
   */
  public function formatEntryAuthor(array $e)
  {
    if (!$this->isValidArrayKey($e, 'author')) return;
    $names = $this->getNames($e['author']);

    return ' <span class="bib-author">'. $names .'</span>'. $this->DOT;
  }

  public static function joinNameInitials($match)
  {
    // Input is "M. J. "
    // Output is "M.J. "
    return str_replace(' ', '', $match[0]) . ' ';
  }
  
  protected function getNames($author)
  {
    // join author initials, if provided
    if ( preg_match_all('#[A-Z]\.\s[A-Z]\.\s#', $author, $matches) ) {
      $author = preg_replace_callback('#[A-Z]\.\s[A-Z]\.\s#', "MyPublication::joinNameInitials", $author);
    }

    // check if author names are limited
    $limit = $this->maxNumAuthors;
    if ($limit < 0) $limit = 0;
    if ($limit == 0) $limit = 99; // display all authors
    // authors can be divided by "and" or "&"
    $authors = preg_split("/\s(and|&)\s/i", $author);
    $numauthors = count($authors);
    if ($numauthors > $limit) {
      $entry = "";
      foreach ($authors as $index => $name) {
        if ($index == $limit) break;
        $entry .= $name . ", ";
      }
      $entry .= '<acronym title="'.($numauthors-$limit).' more ('.$numauthors.' co-authors in total)">et al</acronym>'; // do not put the final dot, since it is added at last
    } else {
      $entry = implode(", ", $authors);
    }

    return $entry;
  }

  /**
   * Applies HTML format to BIB title: <span class="bib-title">
   * @param $e array fields
   */   
  public function formatEntryTitle(array $e)
  {
    if (!$this->isValidArrayKey($e, 'title')) return;

    // convert LaTeX dashes to HTML entities
    $title = $this->fixnumrange($e['title']);

    return ' <span class="bib-title">'. $title .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB journal: <span class="bib-journal">
   * @param $e array fields
   */      
  public function formatEntryJournal(array $e)
  {
    if (!$this->isValidArrayKey($e, 'journal')) return;
    
    return ' <span class="bib-journal">'. $e['journal'] .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB year: <span class="bib-year">
   * @param $e array fields
   */    
  public function formatEntryYear(array $e)
  {
    if (!$this->isValidArrayKey($e, 'year')) return;
    
    return ' <span class="bib-year">'. $e['year'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB volume: <span class="bib-volume">
   * @param $e array fields
   */     
  public function formatEntryVolume(array $e)
  {
    if (!$this->isValidArrayKey($e, 'volume')) return;
    
    return ' Vol. <span class="bib-volume">'. $e['volume'] .'</span>';
  }

  /**
   * Applies HTML format to BIB number: <span class="bib-number">
   * @param $e array fields
   */    
  public function formatEntryNumber(array $e)
  {
    if (!$this->isValidArrayKey($e, 'number')) return;
    
    return '(<span class="bib-number">'. $e['number'] .'</span>)'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB series: <span class="bib-series">
   * @param $e array fields
   */     
  public function formatEntrySeries(array $e)
  {
    if (!$this->isValidArrayKey($e, 'series')) return;
    
    return ' <span class="bib-series">'. $e['series'] .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB booktitle: <span class="bib-booktitle">
   * @param $e array fields
   */   
  public function formatEntryBooktitle(array $e)
  {
    if (!$this->isValidArrayKey($e, 'booktitle')) return;
    
    return ' <span class="bib-booktitle">'. $this->delatex($e['booktitle']) .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB editor: <span class="bib-editor">
   * @param $e array fields
   */  
  public function formatEntryEditor(array $e)
  {
    if (!$this->isValidArrayKey($e, 'editor')) return;

    $names = $this->getNames($e['editor']);

    return ' <span class="bib-editor">'. $names .' (Editors)</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB pages: <span class="bib-pages">
   * @param $e array fields
   */   
  public function formatEntryPages(array $e)
  {
    if (!$this->isValidArrayKey($e, 'pages')) return;

    $parsePage = new PARSEPAGE();
    list($start, $end) = $parsePage->init($e['pages']);
    $entry = $end ? "pp. " . $start . "&ndash;" . $end : "p. " . $start;

    return ' <span class="bib-pages">'. $entry .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB organization: <span class="bib-organization">
   * @param $e array fields
   */  
  public function formatEntryOrganization(array $e)
  {
    if (!$this->isValidArrayKey($e, 'organization')) return;
    
    return ' <span class="bib-organization">'. $e['organization'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB chapter: <span class="bib-chapter">
   * @param $e array fields
   */    
  public function formatEntryChapter(array $e)
  {
    if (!$this->isValidArrayKey($e, 'chapter')) return;
    
    return ' <span class="bib-chapter">'. $e['chapter'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB publisher: <span class="bib-publisher">
   * @param $e array fields
   */  
  public function formatEntryPublisher(array $e)
  {
    if (!$this->isValidArrayKey($e, 'publisher')) return;
    
    return ' <span class="bib-publisher">'. $e['publisher'] .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB edition: <span class="bib-edition">
   * @param $e array fields
   */   
  public function formatEntryEdition(array $e)
  {
    if (!$this->isValidArrayKey($e, 'edition')) return;
    
    return ' <span class="bib-edition">'. $e['edition'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB school: <span class="bib-school">
   * @param $e array fields
   */      
  public function formatEntrySchool(array $e)
  {
    if (!$this->isValidArrayKey($e, 'school')) return;
    
    return ' <span class="bib-school">'. $e['school'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB howpublished: <span class="bib-howpublished">
   * @param $e array fields
   */     
  public function formatEntryHowpublished(array $e)
  {
    if (!$this->isValidArrayKey($e, 'howpublished')) return;
    
    str_link_urls($e['howpublished']);
    return ' <span class="bib-howpublished">'. $e['howpublished'] .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB institution: <span class="bib-institution">
   * @param $e array fields
   */    
  public function formatEntryInstitution(array $e)
  {
    if (!$this->isValidArrayKey($e, 'institution')) return;
    
    return ' <span class="bib-institution">'. $e['institution'] .'</span>'. $this->DOT;
  } 

  /**
   * Applies HTML format to BIB type: <span class="bib-type">
   * @param $e array fields
   */      
  public function formatEntryType(array $e)
  {
    if (!$this->isValidArrayKey($e, 'type')) return;
    
    return ' <span class="bib-type">'. $e['type'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB address: <span class="bib-address">
   * @param $e array fields
   */    
  public function formatEntryAddress(array $e)
  {
    if (!$this->isValidArrayKey($e, 'address')) return;
    
    return ' <span class="bib-address">'. $e['address'] .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB month: <span class="bib-month">
   * @param $e array fields
   */    
  public function formatEntryMonth(array $e)
  {
    if (!$this->isValidArrayKey($e, 'month')) return;

    return ' <span class="bib-month">'. $this->fixnumrange($e['month']) .'</span>'. $this->COMMA;
  }

  /**
   * Applies HTML format to BIB note: <span class="bib-note">
   * @param $e array fields
   */   
  public function formatEntryNote(array $e)
  {
    if (!$this->isValidArrayKey($e, 'note')) return;
    
    // parse hyperref links, or otherwise make URLs clickable
    if (!str_link_hrefs($e['note'])) str_link_urls($e['note']);
    
    return ' <span class="bib-note">'. $e['note'] .'</span>'. $this->DOT;
  }

  /**
   * Applies HTML format to BIB key: <span class="bib-key">
   * @param $e array fields
   */    
  public function formatEntryKey(array $e)
  {
    if (!$this->isValidArrayKey($e, 'key')) return;
    
    return ' <span class="bib-key">'. $e['key'] .'</span>'. $this->DOT;
  }

  /**
   * Ensures that a key exists in array, checking also that it's not empty.
   * @param $e array fields
   * @param $key string array key
   */
  protected function isValidArrayKey(array $e, $key)
  {
    if (array_key_exists($key, $e)) {
      $test = trim($e[$key]);
      return !empty($test);
    } else {
      return false;
    }
  }

  /**
   * Converts LaTeX dashes to HTML entities.
   * @param $text string
   */
  protected function fixnumrange($text)
  {
    return preg_replace('#[--]+[-]#', "&ndash;", $text);
  }

  /**
   * Clears LaTeX chars. This method is not yet complete -- ToDo: parse LaTeX special chars, accents, etc.
   * @param $text string
   */
  protected function delatex($text)
  {
    $patterns = array(
                       '#\\\.#',    // escaped entities
                       '#\{?\}?#',  // brackets
                       '#~#'        // spaces
                     );
    $text = preg_replace($patterns, "", $text);

    return $text;
  }

}
?>
