<?php
// to make this script available standalone:
require_once realpath(dirname(__FILE__).'/../').'/config.php';

// These are global, helper functions
if (DEBUG) {
  error_reporting(E_ALL | E_STRICT);
}

// ignore PHP strict notice if time zone has not been set in php.ini
$defaultTimeZone = @date_default_timezone_get();
if ($defaultTimeZone) {
  $location = $defaultTimeZone;
} else if (ini_get('date.timezone')) {
  $location = ini_get('date.timezone');
} else {
  $location = 'UTC';
}
// set date
date_default_timezone_set($location);

// NEW, for PHP >= 8
function build_sorter($key) {
  return function ($a, $b) use ($key) {
    if ($a[$key] == $b[$key]) return 0;
    return $a[$key] < $b[$key] ? -1 : 1;
  };
}

/**
 * Prints HTML content in browser
 * @param $content string HTML text
 */
function print_line($content)
{
  echo $content;
  // adding a newline allows a better source code reading, but browser do prefer the page in one line (will load faster)
  if (DEBUG) echo PHP_EOL;
}

/**
 * Ensures that a web address begins with 'http://'
 * @param $uri  string  web address
 * @return      string  Sanitized web address
 */
function url_check_protocol($uri)
{
  if (strpos($uri, "http://") === false) {
    $uri = "http://" . $uri;
  }

  return $uri;
}

/**
 * Retrieves the URL of the current page.
 * @return      string  URL
 */
function url_get_base() 
{
  $protocol = "http" . ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != "off") ? "s" : null) . "://";
  $host = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];

  $base = $_SERVER['REQUEST_URI'];
  $baseparts = explode("/", $base);
  unset($baseparts[count($baseparts)-1]);
  $base = implode("/", $baseparts);

  return $protocol.$host.$base;
}

/**
 * Extract URLs from plain text and wrap them in (clickable) links
 * @param $str  string  input text (is passed as reference, not as value)
 */
function str_link_urls(&$str)
{
  $str = preg_replace('/[a-z]+:\/\/\S+/', '<a href="$0">$0</a>', $str);
  return true;
}
  
/**
 * Returns useful file info.
 * @param $filename string file name
 * @param $attribute string file attribute: "dirname", "basename", "extension", or "filename" (default)
 * @return The attribute file info
 */
function file_info($filename, $attribute = "filename")
{
  $file = strtolower($filename);
  $info = pathinfo($file);
  if (!array_key_exists($attribute, $info)) return false;

  return $info[$attribute];
}

/** 
 * Converts a string to filename (only alphanumeric and dashed chars).
 * This is used for generating long, SEO friendly URLs.
 * @param $str  string  input text
 * @return  string  Normalized file name
 */
function str_to_filename($str) 
{
  // remove non alphanumeric chars (spaces and dashes are preserved)
  $str = preg_replace('/[^a-z0-9A-Z\s-]+/', '', strtolower($str));
  // now convert spaces to dashes
  $str = str_replace(" ", "-", $str);
  
  return $str;
}

/** 
 * Convert filename to string (dashes to spaces).
 * @param $string string  filename
 * @return  string  name
 */
function filename_to_str($str) 
{
  $str = str_replace("-", " ", $str);
  
  return $str;
}

/** 
 * Checks if a string begins with a certain prefix.
 * @param   string    $string  Input string
 * @param   string    $prefix  Prefix to search in string
 * @return  boolean   TRUE on success or FALSE on failure
 */
function str_startswith($string, $prefix)
{
   return strncmp($string, $prefix, strlen($prefix)) == 0;
}

/** 
 * Converts a CSV string into a PHP array. Values are trimmed and lowercased.
 * @param string $str  Input string
 * @return array converted array
 */
function csv2array($str) 
{
  $a = explode(",", $str);
  $a = array_map('trim', $a);
  $a = array_map('strtolower', $a);

  return $a;
}

/** 
 * Do a raw POST request to given URL.
 * @param string $url  URL to post data
 * @param array $data  post data 
 * @param string $referer  Referer URL
 * @return array result of POST request
 */
function post_request($url, $data, $referer = "") 
{ 
  $RN = "\r\n";
  // convert the data array into URL Parameters like a=b&foo=bar etc.
  $data = http_build_query($data);
  // parse the given URL
  $url = parse_url($url);
  if ($url['scheme'] != 'http') { // scheme error 
    return array(
      'status' => 1, 
      'error' => "Only HTTP request are supported"
    );
  }
  // extract host and path
  $host = $url['host'];
  $path = $url['path'];
  // open a socket connection on port 80 - timeout: 30 sec
  $fp = fsockopen($host, 80, $errno, $errstr, 30);
  if ($fp) {
    // send the request headers
    $request = "POST $path HTTP/1.1".$RN;
    $request.= "Host: $host".$RN;
    // add referrer
    if (!empty($referer)) $request.= "Referer: $referer".$RN;
    // mandatory fields
    $request.= "Content-type: application/x-www-form-urlencoded".$RN;
    $request.= "Content-Length: ".strlen($data).$RN;
    $request.= "Connection: Close".$RN.$RN;
    $request.= $data;
    // send
    fputs($fp, $request);
    // receive the results of the request
    $result = ""; 
    while(!feof($fp)) {
      $result .= fgets($fp, 128);
    }
  } else { // socket error
    return array(
      'status' => 2, 
      'error' => "$errstr ($errno)"
    );
  }
  fclose($fp);
  // split the result header from the content
  list($header, $content) = explode($RN.$RN, $result, 2);
  // return as structured array
  return array(
    'status' => 0,
    'header' => $header,
    'content' => $content
  );
}
?>
